<?php


class APPMAKER_WC_General_hooks {
	/**
	 * Holds the values to be used in the fields callbacks
	 *
	 * @var object
	 */
	private $options;

	/**
	 * Start up
	 */
	public function __construct() {
		require_once( APPMAKER_WC::$root . '/lib/vendor/fcm/class-appmaker-wc-fcm-helper.php' );

		if ( ! empty( $_GET['payment_from_app'] ) ) {
			add_action( 'wp_head', array( $this, 'hook_stripe_enable_headers' ) );
			add_action( 'wp_footer', array( $this, 'hook_payment_footer' ) );
		}
		add_action( 'woocommerce_order_status_completed', array( $this, 'appmaker_wc_order_status_changed' ), 10, 1 );
		add_action( 'woocommerce_order_status_processing', array( $this, 'appmaker_wc_order_status_changed' ), 10, 1 );
		add_action( 'woocommerce_order_status_refunded', array( $this, 'appmaker_wc_order_status_changed' ), 10, 1 );
		add_action( 'woocommerce_order_status_cancelled', array( $this, 'appmaker_wc_order_status_changed' ), 10, 1 );
		add_action( 'woocommerce_order_status_on-hold', array( $this, 'appmaker_wc_order_status_changed' ), 10, 1 );
		add_action( 'woocommerce_order_status_failed', array( $this, 'appmaker_wc_order_status_changed' ), 10, 1 );
		add_action( 'woocommerce_order_status_pending', array( $this, 'appmaker_wc_order_status_changed' ), 10, 1 );
		add_action( 'woocommerce_loaded', array( $this, 'load_persistent_cart' ), 10, 1 );

	}

	/**
	 * Order change callback function
	 *
	 * @param int $order_id order id.
	 */
	public function appmaker_wc_order_status_changed( $order_id ) {
	 	$fcm_key = APPMAKER_WC::$api->get_settings( 'fcm_server_key' );
		$order   = new WC_Order( $order_id );
		if ( ! empty( $fcm_key ) && APPMAKER_WC::$api->get_settings( 'enable_order_push', true ) ) {
			$fcm     = new Appmaker_WC_FCM_Helper( $fcm_key );
			$user_id = self::get_property($order,'user_id');

			if ( class_exists( 'WC_Seq_Order_Number' ) ) {
				$display_order_id = self::get_property($order,'order_number');
			}  else {
                $display_order_id = self::get_id($order);
            }

			if ( ! empty( $user_id ) && get_user_meta( $user_id, 'appmaker_wc_user_login_from_app' ) ) {
				sprintf( __( 'Order updated #%s', 'appmaker-woocommerce-mobile-app-manager' ), $display_order_id );
				$fcm->setTopic( "user-$user_id" )
				    ->setMessage( sprintf( __( 'Order updated #%s', 'appmaker-woocommerce-mobile-app-manager' ), $display_order_id ),
					sprintf( __( 'Order status changed to %s', 'appmaker-woocommerce-mobile-app-manager' ), $order->get_status() ))
				    ->setAction( array(
					    'type'   => 'OPEN_ORDER',
					    'params' => array(
						    'orderId' => $order_id,
					    ),
				    ) )
				    ->send();
			}
		}

	}

	public function hook_stripe_enable_headers() {
		$output = '<style> .stripe_checkout_app { height: 580px !important; } </style>
                   <meta name="mobile-web-app-capable" content="yes">
                   <meta name="viewport" content="width=device-width, initial-scale=1.0">';
		echo $output;
	}

	public function hook_payment_footer() {
		$gateway = isset( $_GET['payment_gateway'] ) ? $_GET['payment_gateway'] : '';
		$output  = '
				<script type="text/javascript">
				window.onload = function() { 
					setTimeout(function(){
				';
		if ( ! empty( $gateway ) ) {
			$output .= "\n\t\t" . 'document.getElementById("payment_method_' . $gateway . '").checked = true;';
			$output .= "\n\t\t" . 'document.getElementById("payment_method_' . $gateway . '").click();';
		}
		$output .= "\n\t\t" . 'if(document.getElementById("terms") != null ) {';
		$output .= "\n\t\t" . 'document.getElementById("terms").checked = true;
			}
		';
		if ( isset( $gateway ) && ! in_array( $gateway, apply_filters( 'appmaker_wc_checkout_skip_click', array( 'square' ) ) ) ) {
			$output .= "\n\t\t" . '
			setTimeout(function(){
			if(document.getElementById("CBAWidgets1") != null){
				document.getElementById("CBAWidgets1").click();			
			} else {
				document.getElementById("place_order").click();
			}			
			},1000);
			';
		} else {
			$output .= "\n" . 'document.getElementById("payment_method_' . $gateway . '").scrollIntoView();';
		}
		$output .= '
					},500);
			';
		$output .= '
			}
			</script>
		  ';

		echo $output;
	}

	/**
	 * Load the persistent cart make cart sync with app
	 *
	 * @return void|bool
	 */
	public function load_persistent_cart() {
		global $current_user;

		if ( ! $current_user ) {
			return false;
		}

		$saved_cart = get_user_meta( $current_user->ID, '_woocommerce_persistent_cart', true );

		if ( $saved_cart && is_array( $saved_cart ) && isset( $saved_cart['cart'] ) ) {
			WC()->session->set( 'cart', $saved_cart['cart'] );
		}

		return true;
	}

    public static function get_id( $object ) {
        if ( method_exists( $object, 'get_id' ) ) {
            return $object->get_id();
        } else {
            return $object->id;
        }
    }

    static function get_property( $object, $property ) {
        if ( method_exists( $object, 'get_'.$property ) ) {
            return call_user_func(array($object, 'get_'.$property));
        } else {
            return $object->{$property};
        }
    }
}

new APPMAKER_WC_General_hooks();
