<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class APPMAKER_WC_Product_Addons {

	public function __construct() {
        add_filter( 'appmaker_wc_product_fields', array( $this, 'product_fields' ), 2, 2 );
	}

	/**
	 * @param array $fields
	 * @param WC_Product $product
	 *
	 * @return array|mixed|void
	 */
	public function product_fields($fields,$product ) {
		$product_addons = WC_Product_Addons_Helper::get_product_addons( $product->get_id() );
		$fields         = array();
		foreach ( $product_addons as $addon => $value ) {
		   if($value['type']=='multiple_choice'){
		       $value['type']= 'select';
		       $product_addons[$addon]['type']='select';
            }
			if( ! in_array( $value['type'],
				array(
					'select',
					'radiobutton',
					'custom',
					'custom_letters_or_digits',
					'custom_letters_only',
					'custom_textarea',
					'custom_digits_only',
					'custom_email',
					'custom_price',
					'input_multiplier',
					'checkbox',


			), true )
			) {
				continue;
			}

			$required = ( $value['required'] === 1 || $value['required'] === '1' );
			if ( $value['type'] === 'select' || $value['type'] === 'radiobutton' ) {
				$key = 'addon-' . sanitize_title( $value['field_name'] );
				$fields[ $key ]['required'] = $required;
				$fields[ $key ]['label'] = $value['name'];
				$fields[ $key ]['type'] = 'select';
				$fields[ $key ]['options'] = array();
				$loop = 0;
				foreach ( $value['options'] as $option ) {
					$loop ++;
					if ( $value['type'] === 'radiobutton' ) {
						$option_value = sanitize_title( $option['label'] );
					} else {
						$option_value = sanitize_title( $option['label'] ) . '-' . $loop;
					}
					$fields[ $key ]['options'][ $option_value ] = $option['label'];
					if ( ! empty( $option['price'] ) && $option['price'] > 0 ) {
						$fields[ $key ]['options'][ $option_value ] .= ' (' . APPMAKER_WC_Helper::get_display_price( $option['price'] ) . ') ';
					}
				}
			} else {
				foreach ( $value['options'] as $id => $option ) {
					if ( $value['type'] === 'checkbox' ) {
						$key = 'addon-' . sanitize_title( $value['field-name'] ) . "[$id]";
					} else {
						$option_key = empty( $option['label'] ) ? $id : sanitize_title( $option['label'] );
						$key = 'addon-' . sanitize_title( $value['field-name'] ) . '[' . $option_key . ']';
					}
					$fields[ $key ]['required'] = $required;
					$fields[ $key ]['label']    = ! empty( $option['label'] ) ? $option['label'] : $value['name'];
					if ( ! empty( $option['price'] ) && $option['price'] > 0 ) {
						$fields[ $key ]['label'] .= ' (' . APPMAKER_WC_Helper::get_display_price( $option['price'] ) . ') ';
					}
					switch ( $value['type'] ) {
						case 'custom' :
						case 'custom_letters_or_digits' :
						case 'custom_letters_only' :
							$fields[ $key ]['type'] = 'text';
							break;
						case 'custom_textarea' :
							$fields[ $key ]['type'] = 'textarea';
							break;
						case 'custom_digits_only' :
							$fields[ $key ]['type'] = 'number';
							break;
						case 'custom_email' :
							$fields[ $key ]['type'] = 'email';
							break;
						case 'custom_price' :
						case 'input_multiplier' :
							$fields[ $key ]['type'] = 'number';
							break;
						case 'checkbox' :
							$fields[ $key ]['type'] = 'checkbox';
							break;
					}
				}
			}
		}
		$fields = APPMAKER_WC_Dynamic_form::get_fields( $fields, 'product' );

		return $fields;
	}

}

new APPMAKER_WC_Product_Addons();
